package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.ConditionalExpression;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.ExpressionContext;
import org.eclipse.jdt.internal.compiler.ast.Invocation;
import org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching;
import org.eclipse.jdt.internal.compiler.ast.SwitchStatement;
import org.eclipse.jdt.internal.compiler.ast.UnaryExpression;
import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public abstract class Statement extends ASTNode {
  public static final int NOT_COMPLAINED = 0;
  
  public static final int COMPLAINED_FAKE_REACHABLE = 1;
  
  public static final int COMPLAINED_UNREACHABLE = 2;
  
  protected static boolean isKnowDeadCodePattern(Expression expression) {
    if (expression instanceof UnaryExpression)
      expression = ((UnaryExpression)expression).expression; 
    if (expression instanceof org.eclipse.jdt.internal.compiler.ast.Reference)
      return true; 
    return false;
  }
  
  public abstract FlowInfo analyseCode(BlockScope paramBlockScope, FlowContext paramFlowContext, FlowInfo paramFlowInfo);
  
  public boolean doesNotCompleteNormally() {
    return false;
  }
  
  public boolean completesByContinue() {
    return false;
  }
  
  protected void analyseArguments(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, MethodBinding methodBinding, Expression[] arguments) {
    if (arguments != null) {
      CompilerOptions compilerOptions = currentScope.compilerOptions();
      if (compilerOptions.sourceLevel >= 3342336L && methodBinding.isPolymorphic())
        return; 
      boolean considerTypeAnnotations = currentScope.environment().usesNullTypeAnnotations();
      boolean hasJDK15NullAnnotations = (methodBinding.parameterNonNullness != null);
      int numParamsToCheck = methodBinding.parameters.length;
      int varArgPos = -1;
      TypeBinding varArgsType = null;
      boolean passThrough = false;
      if (considerTypeAnnotations || hasJDK15NullAnnotations)
        if (methodBinding.isVarargs()) {
          varArgPos = numParamsToCheck - 1;
          if (numParamsToCheck == arguments.length) {
            varArgsType = methodBinding.parameters[varArgPos];
            TypeBinding lastType = (arguments[varArgPos]).resolvedType;
            if (lastType == TypeBinding.NULL || (
              varArgsType.dimensions() == lastType.dimensions() && 
              lastType.isCompatibleWith(varArgsType)))
              passThrough = true; 
          } 
          if (!passThrough)
            numParamsToCheck--; 
        }  
      if (considerTypeAnnotations) {
        for (int i = 0; i < numParamsToCheck; i++) {
          TypeBinding expectedType = methodBinding.parameters[i];
          Boolean specialCaseNonNullness = hasJDK15NullAnnotations ? methodBinding.parameterNonNullness[i] : null;
          analyseOneArgument18(currentScope, flowContext, flowInfo, expectedType, arguments[i], 
              specialCaseNonNullness, (methodBinding.original()).parameters[i]);
        } 
        if (!passThrough && varArgsType instanceof ArrayBinding) {
          TypeBinding expectedType = ((ArrayBinding)varArgsType).elementsType();
          Boolean specialCaseNonNullness = hasJDK15NullAnnotations ? methodBinding.parameterNonNullness[varArgPos] : null;
          for (int j = numParamsToCheck; j < arguments.length; j++)
            analyseOneArgument18(currentScope, flowContext, flowInfo, expectedType, arguments[j], 
                specialCaseNonNullness, (methodBinding.original()).parameters[varArgPos]); 
        } 
      } else if (hasJDK15NullAnnotations) {
        for (int i = 0; i < numParamsToCheck; i++) {
          if (methodBinding.parameterNonNullness[i] == Boolean.TRUE) {
            TypeBinding expectedType = methodBinding.parameters[i];
            Expression argument = arguments[i];
            int nullStatus = argument.nullStatus(flowInfo, flowContext);
            if (nullStatus != 4)
              flowContext.recordNullityMismatch(currentScope, argument, argument.resolvedType, expectedType, flowInfo, nullStatus, null); 
          } 
        } 
      } 
    } 
  }
  
  void analyseOneArgument18(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, TypeBinding expectedType, Expression argument, Boolean expectedNonNullness, TypeBinding originalExpected) {
    if (argument instanceof ConditionalExpression && argument.isPolyExpression()) {
      ConditionalExpression ce = (ConditionalExpression)argument;
      ce.internalAnalyseOneArgument18(currentScope, flowContext, expectedType, ce.valueIfTrue, flowInfo, ce.ifTrueNullStatus, expectedNonNullness, originalExpected);
      ce.internalAnalyseOneArgument18(currentScope, flowContext, expectedType, ce.valueIfFalse, flowInfo, ce.ifFalseNullStatus, expectedNonNullness, originalExpected);
      return;
    } 
    int nullStatus = argument.nullStatus(flowInfo, flowContext);
    internalAnalyseOneArgument18(currentScope, flowContext, expectedType, argument, flowInfo, 
        nullStatus, expectedNonNullness, originalExpected);
  }
  
  void internalAnalyseOneArgument18(BlockScope currentScope, FlowContext flowContext, TypeBinding expectedType, Expression argument, FlowInfo flowInfo, int nullStatus, Boolean expectedNonNullness, TypeBinding originalExpected) {
    int statusFromAnnotatedNull = (expectedNonNullness == Boolean.TRUE) ? nullStatus : 0;
    NullAnnotationMatching annotationStatus = NullAnnotationMatching.analyse(expectedType, argument.resolvedType, nullStatus);
    if (!annotationStatus.isAnyMismatch() && statusFromAnnotatedNull != 0)
      expectedType = originalExpected; 
    if (statusFromAnnotatedNull == 2) {
      currentScope.problemReporter().nullityMismatchingTypeAnnotation(argument, argument.resolvedType, expectedType, annotationStatus);
    } else if (annotationStatus.isAnyMismatch() || (statusFromAnnotatedNull & 0x10) != 0) {
      if (!expectedType.hasNullTypeAnnotations() && expectedNonNullness == Boolean.TRUE) {
        LookupEnvironment env = currentScope.environment();
        expectedType = env.createAnnotatedType(expectedType, new AnnotationBinding[] { env.getNonNullAnnotation() });
      } 
      flowContext.recordNullityMismatch(currentScope, argument, argument.resolvedType, expectedType, flowInfo, nullStatus, annotationStatus);
    } 
  }
  
  protected void checkAgainstNullTypeAnnotation(BlockScope scope, TypeBinding requiredType, Expression expression, FlowContext flowContext, FlowInfo flowInfo) {
    if (expression instanceof ConditionalExpression && expression.isPolyExpression()) {
      ConditionalExpression ce = (ConditionalExpression)expression;
      internalCheckAgainstNullTypeAnnotation(scope, requiredType, ce.valueIfTrue, ce.ifTrueNullStatus, flowContext, flowInfo);
      internalCheckAgainstNullTypeAnnotation(scope, requiredType, ce.valueIfFalse, ce.ifFalseNullStatus, flowContext, flowInfo);
      return;
    } 
    int nullStatus = expression.nullStatus(flowInfo, flowContext);
    internalCheckAgainstNullTypeAnnotation(scope, requiredType, expression, nullStatus, flowContext, flowInfo);
  }
  
  private void internalCheckAgainstNullTypeAnnotation(BlockScope scope, TypeBinding requiredType, Expression expression, int nullStatus, FlowContext flowContext, FlowInfo flowInfo) {
    NullAnnotationMatching annotationStatus = NullAnnotationMatching.analyse(requiredType, expression.resolvedType, null, null, nullStatus, expression, NullAnnotationMatching.CheckMode.COMPATIBLE);
    if (annotationStatus.isDefiniteMismatch()) {
      scope.problemReporter().nullityMismatchingTypeAnnotation(expression, expression.resolvedType, requiredType, annotationStatus);
    } else {
      if (annotationStatus.wantToReport())
        annotationStatus.report((Scope)scope); 
      if (annotationStatus.isUnchecked())
        flowContext.recordNullityMismatch(scope, expression, expression.resolvedType, requiredType, flowInfo, nullStatus, annotationStatus); 
    } 
  }
  
  public void branchChainTo(BranchLabel label) {}
  
  public boolean breaksOut(char[] label) {
    return (new Object(this, label))




















      
      .breaksOut();
  }
  
  public boolean continuesAtOuterLabel() {
    return (new Object(this))









      
      .continuesAtOuterLabel();
  }
  
  public int complainIfUnreachable(FlowInfo flowInfo, BlockScope scope, int previousComplaintLevel, boolean endOfBlock) {
    if ((flowInfo.reachMode() & 0x3) != 0) {
      if ((flowInfo.reachMode() & 0x1) != 0)
        this.bits &= Integer.MAX_VALUE; 
      if (flowInfo == FlowInfo.DEAD_END) {
        if (previousComplaintLevel < 2) {
          scope.problemReporter().unreachableCode(this);
          if (endOfBlock)
            scope.checkUnclosedCloseables(flowInfo, null, null, null); 
        } 
        return 2;
      } 
      if (previousComplaintLevel < 1) {
        scope.problemReporter().fakeReachable(this);
        if (endOfBlock)
          scope.checkUnclosedCloseables(flowInfo, null, null, null); 
      } 
      return 1;
    } 
    return previousComplaintLevel;
  }
  
  public void generateArguments(MethodBinding binding, Expression[] arguments, BlockScope currentScope, CodeStream codeStream) {
    if (binding.isVarargs()) {
      TypeBinding[] params = binding.parameters;
      int paramLength = params.length;
      int varArgIndex = paramLength - 1;
      for (int i = 0; i < varArgIndex; i++)
        arguments[i].generateCode(currentScope, codeStream, true); 
      ArrayBinding varArgsType = (ArrayBinding)params[varArgIndex];
      ArrayBinding codeGenVarArgsType = (ArrayBinding)binding.parameters[varArgIndex].erasure();
      int elementsTypeID = (varArgsType.elementsType()).id;
      int argLength = (arguments == null) ? 0 : arguments.length;
      if (argLength > paramLength) {
        codeStream.generateInlinedValue(argLength - varArgIndex);
        codeStream.newArray(codeGenVarArgsType);
        for (int j = varArgIndex; j < argLength; j++) {
          codeStream.dup();
          codeStream.generateInlinedValue(j - varArgIndex);
          arguments[j].generateCode(currentScope, codeStream, true);
          codeStream.arrayAtPut(elementsTypeID, false);
        } 
      } else if (argLength == paramLength) {
        TypeBinding lastType = (arguments[varArgIndex]).resolvedType;
        if (lastType == TypeBinding.NULL || (
          varArgsType.dimensions() == lastType.dimensions() && 
          lastType.isCompatibleWith((TypeBinding)codeGenVarArgsType))) {
          arguments[varArgIndex].generateCode(currentScope, codeStream, true);
        } else {
          codeStream.generateInlinedValue(1);
          codeStream.newArray(codeGenVarArgsType);
          codeStream.dup();
          codeStream.generateInlinedValue(0);
          arguments[varArgIndex].generateCode(currentScope, codeStream, true);
          codeStream.arrayAtPut(elementsTypeID, false);
        } 
      } else {
        codeStream.generateInlinedValue(0);
        codeStream.newArray(codeGenVarArgsType);
      } 
    } else if (arguments != null) {
      for (int i = 0, max = arguments.length; i < max; i++)
        arguments[i].generateCode(currentScope, codeStream, true); 
    } 
  }
  
  public abstract void generateCode(BlockScope paramBlockScope, CodeStream paramCodeStream);
  
  public boolean isBoxingCompatible(TypeBinding expressionType, TypeBinding targetType, Expression expression, Scope scope) {
    if (scope.isBoxingCompatibleWith(expressionType, targetType))
      return true; 
    return (expressionType.isBaseType() && 
      !targetType.isBaseType() && 
      !targetType.isTypeVariable() && 
      (scope.compilerOptions()).sourceLevel >= 3211264L && (
      targetType.id == 26 || targetType.id == 27 || targetType.id == 28) && 
      expression.isConstantValueOfTypeAssignableToType(expressionType, scope.environment().computeBoxingType(targetType)));
  }
  
  public boolean isEmptyBlock() {
    return false;
  }
  
  public boolean isValidJavaStatement() {
    return true;
  }
  
  public StringBuffer print(int indent, StringBuffer output) {
    return printStatement(indent, output);
  }
  
  public abstract StringBuffer printStatement(int paramInt, StringBuffer paramStringBuffer);
  
  public abstract void resolve(BlockScope paramBlockScope);
  
  public Constant resolveCase(BlockScope scope, TypeBinding testType, SwitchStatement switchStatement) {
    resolve(scope);
    return Constant.NotAConstant;
  }
  
  public TypeBinding invocationTargetType() {
    return null;
  }
  
  public TypeBinding expectedType() {
    return invocationTargetType();
  }
  
  public ExpressionContext getExpressionContext() {
    return ExpressionContext.VANILLA_CONTEXT;
  }
  
  protected MethodBinding findConstructorBinding(BlockScope scope, Invocation site, ReferenceBinding receiverType, TypeBinding[] argumentTypes) {
    MethodBinding ctorBinding = scope.getConstructor(receiverType, argumentTypes, (InvocationSite)site);
    resolvePolyExpressionArguments(site, ctorBinding, argumentTypes, scope);
    return ctorBinding;
  }
}
